﻿local L = AceLibrary("AceLocale-2.2"):new("GridStatusPvP")
local GridRoster = GridRoster

local PlayerFaction = UnitFactionGroup("player")

L:RegisterTranslations("enUS", function() return {
	["PvP"] = true,
	["<PvP>"] = true,
	["<PvP-FFA>"] = true,
	["PvP alert"] = true,
} end)

L:RegisterTranslations("deDE", function() return {
	["PvP"] = "PvP",
	["<PvP>"] = "<PvP>",
	["<PvP-FFA>"] = "<PvP-FFA>",
	["PvP alert"] = "PvP Alarm",
} end)

L:RegisterTranslations("zhCN", function() return {
	["PvP"] = "PvP",
	["<PvP>"] = "<PvP>",
	["<PvP-FFA>"] = "<PvP-自由>",
	["PvP alert"] = "PvP 警报",
} end)

L:RegisterTranslations("zhTW", function() return {
	["PvP"] = "PvP",
	["<PvP>"] = "<PvP>",
	["<PvP-FFA>"] = "<PvP-自由對戰>",
	["PvP alert"] = "PvP 警報",
} end)

GridStatusPvP = GridStatus:NewModule("GridStatusPvP")
local GridStatusPvP = GridStatusPvP
GridStatusPvP.menuName = L["PvP"]

GridStatusPvP.defaultDB = {
	debug = false,
	alert_pvp = {
		text =  L["<PvP>"],
		enable = false,
		color = { r = 0.0, g = 1.0, b = 1.0, a = 0.75 },
		priority = 20,
		range = false,
	},
}

GridStatusPvP.options = false

function GridStatusPvP:OnInitialize()
	self.super.OnInitialize(self)

	self:RegisterStatus("alert_pvp", L["PvP alert"], nil, true)
end

function GridStatusPvP:OnEnable()
	self.debugging = self.db.profile.debug
	self:Debug("OnEnable")

	self.super.OnEnable(self)
end

function GridStatusPvP:Reset()
	self.super.Reset(self)

	self:UnregisterStatus("alert_pvp")
	self:RegisterStatus("alert_pvp", L["PvP alert"], nil, true)
	self:UpdateAllUnits()
end


function GridStatusPvP:OnStatusEnable(status)
	self:RegisterEvent("UNIT_FACTION", "UpdateUnit")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("Grid_UnitJoined")

	self:UpdateAllUnits()
end

function GridStatusPvP:OnStatusDisable(status)
	self.core:SendStatusLostAllUnits(status)

	self:UnregisterEvent("UNIT_FACTION")
	self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
	self:UnregisterEvent("Grid_UnitJoined")
end

local instanceType
function GridStatusPvP:ZONE_CHANGED_NEW_AREA()
	self:Debug("ZONE_CHANGED_NEW_AREA")
	instanceType = select(2, IsInInstance())
	if (instanceType == "pvp" or instanceType == "arena") then
		-- We are in bg/arena: clear all pvp statuses
		self.core:SendStatusLostAllUnits("alert_pvp")
	else
		-- We are NOT in bg/arena: check all units for pvp
		self:UpdateAllUnits()
	end
end

function GridStatusPvP:UpdateUnit(unitid)
	-- if we are in bg/arena: return -> do nothing
	if (instanceType == "pvp" or instanceType == "arena") then
		return
	end
	if (string.find(unitid, "pet")) then
		return
	end

	if (UnitIsPVP(unitid) or UnitIsPVPFreeForAll(unitid)) then
		self:UnitHasPvP(unitid)
	else
		self.core:SendStatusLost(UnitGUID(unitid), "alert_pvp")
	end
end

function GridStatusPvP:Grid_UnitJoined(guid, unitid)
	-- if we are in bg/arena: do not update status if someone join group
	if (instanceType == "pvp" or instanceType == "arena") then
		return
	end

	self:UpdateUnit(unitid)
end

function GridStatusPvP:UpdateAllUnits()
	for guid, unitid in GridRoster:IterateRoster() do
		if (UnitIsPVP(unitid) or UnitIsPVPFreeForAll(unitid)) then
			self:UnitHasPvP(unitid)
		else
			self.core:SendStatusLost(guid, "alert_pvp")
		end
	end
end

function GridStatusPvP:UnitHasPvP(unitid)
	local settings = self.db.profile.alert_pvp
	local text = L["<PvP>"]
	local icon = "Interface\\AddOns\\GridStatusPvP\\icons\\PvP-Horde.tga"
	if (PlayerFaction == "Alliance") then
		icon = "Interface\\AddOns\\GridStatusPvP\\icons\\PvP-Alliance.tga"
	end
	if (UnitIsPVPFreeForAll(unitid)) then
		text = L["<PvP-FFA>"]
		icon = "Interface\\AddOns\\GridStatusPvP\\icons\\PvP-FFA.tga"
	end
	self.core:SendStatusGained(UnitGUID(unitid), "alert_pvp",
		settings.priority,
		(settings.range and 40),
		settings.color,
		text,
		nil,
		nil,
		icon)
end